# File name: KookatimeRX.py
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 30 July 2018
# Date last modified: 31 December 2018
# MicroPython Version: 1.9.4 for the Kookaberry
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# Program to read a time tuple from a radio message and update the real time clock in the Kookaberry
#------------------------------------------
# Dependencies:
# I/O ports and peripherals: Nil
# /lib files: Timeconfig.py, doomsday.py, Kapputils.py
# /root files: KappConfig.txt
# Other dependencies: Nil
# Complementary apps: KookaTimeTx.py
#------------------------------------------



import doomsday    # day of week from date program
import os, time, json, machine, kooka, fonts
from Kapputils import config    # utility to read the configuration file
    
disp = kooka.display    # instantiate the display
rtc = machine.RTC()    # instantiate the in-built real time clock
params = config('Kappconfig.txt')   # read the configuration file
# set up the radio for later use
kooka.radio.enable()
chan = int(params['CHANNEL'])      # use channel from the configuration file
baud = int(params['BAUD'])      # use data rate from the configuration file
pwr = int(params['POWER'])      # use transmit power from the configuration file

kooka.radio.config(channel=chan, data_rate=baud, power=pwr) # set up the radio

rtime = [0]*6   # Kookaberry time tuple [YYYY,MM,DD,HH,MM,SS]
ftime = [0]*8   # Kookaberry time tuple [YYYY,MM,DD,WD,HH,MM,SS,SUBS]
ktime = [0]*8   # Kookaberry time tuple [YYYY,MM,DD,WD,HH,MM,SS,SUBS]
months = ["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]

# Display the clock and facilitate manual correction
while not kooka.button_a.was_pressed():
    disp.fill(0)
    disp.setfont(fonts.mono8x8)
    disp.text('Real Time Clock', 0, 6)
    disp.setfont(fonts.mono6x7)
    disp.text('A-exit', 0, 60)
    ktime = rtc.datetime()  # Refresh the time
    for i in range(0,7): ftime[i] = ktime[i]
    disp.setfont(fonts.mono8x8)
    disp.text('%s' % doomsday.calc_dow(ftime[2],ftime[1],ftime[0]), 12, 20)    # Display Weekday
    disp.text('%2d-%s-%d' % (ftime[2], months[ftime[1]-1], ftime[0]),12, 30)   #Display date
    disp.text('%02d:%02d:%02d' % (ftime[4], ftime[5], ftime[6]),12, 40)   #Display time
    disp.show()
    msg = kooka.radio.receive()    # Check for radio messages
    if msg:
        print(msg)
        if len(msg) == 21 and msg[1].isdigit() and msg[2].isdigit() and msg[3].isdigit() and msg[4].isdigit():  # is a time update so process
            rtime = json.loads(msg)
            for i in range(0,4): ftime[i] = rtime[i]    # transfer date
            for i in range(3,6): ftime[i+1] = rtime[i]    # transfer time
            ftime[3] = doomsday.dow_index(ftime[2],ftime[1],ftime[0])    # update day of week 
            rtc.datetime(tuple((ftime[0],ftime[1],ftime[2],ftime[3],ftime[4],ftime[5],ftime[6],0))) # Update the RTC
        else:  pass  # not a time update so ignore      

    time.sleep_ms(100)
# Clean up and exit
kooka.radio.disable()