# File name: Pixel.py
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 10 February 2020
# Date last modified: 10 february 2020
# MicroPython Version: 1.12 for the Kookaberry V4-05 from 2020-08-08 firmware update
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# Demonstrates hue and luminance on a Neopixel
# Three sliders for Luminance, Red, Green and Blue are provided
# The sliders are selected in turn by pressing button B
# The selected slider is adjusted down and up by pressing the C and D buttons repectively
# The Neopixel shows the results for hue and luminance (brightness)
# Note that luminance acts as a dimmer on the hue sliders.
# e.g.  Brightness = luminance x RGB values
#------------------------------------------
# Dependencies:
# I/O ports and peripherals: P2 for NeoPixels
# /lib files: neopixel.py
# /root files: Nil
# Other dependencies: NeoPixel firmware update 2019-08-08
# Complementary apps:
#------------------------------------------

# Begin code
import time, kooka, machine, math, fonts, neopixel, framebuf

# Initial conditions
np_plug = 'P2'
max_lum = 64    # limit upper intensity of LEDs
pixels_in_array = 1
black = (0,0,0)
modes = ['Lum','Red','Grn','Blu']
np_vals = [50,50,50,50]
np_mins = [1,1,1,1]
np_maxs = [100,100,100,100]
np_y = [18,28,38,48]
mode_ptr = 0
rgb = [0,0,0]


# Set up objects
np = neopixel.NeoPixel(machine.Pin(np_plug), pixels_in_array)
np.fill(black)
np.write()

disp = kooka.display

# Function draws a slider bar
def slider(display,x,y,len,cursor):
        display.line(x, y-2, x+len, y-2, 1)
        display.blit(framebuf.FrameBuffer(bytearray(b"\x18\x34\x72\x72\x72\x72\x34\x18"), 8, 8, framebuf.MONO_HMSB), int(x+(len-4)*cursor/100), y-6)


# The main loop begins here
while not kooka.button_a.was_pressed():
# Display the static text
    disp.fill(0)
    disp.setfont(fonts.mono8x8)
    disp.text("Pixel", 0, 6)
    disp.setfont(fonts.mono6x7)
    disp.text('Skt=%s' % np_plug, 80, 6)
    disp.text('A-exit  C-  D+  B-sel', 0, 60)
# Adjust the parameters using the C and D keys
    if kooka.button_c.is_pressed(): 
        np_vals[mode_ptr] = max(np_mins[mode_ptr],np_vals[mode_ptr]-1)
    if kooka.button_d.is_pressed(): 
        np_vals[mode_ptr] = min(np_maxs[mode_ptr],np_vals[mode_ptr]+1)
    if kooka.button_b.was_pressed(): 
        mode_ptr += 1
        if mode_ptr > len(np_vals)-1: mode_ptr = 0
# Display the parameters
    for i in range(0,len(modes)):
        disp.text('%s %d%%' % (modes[i],np_vals[i]), 0, np_y[i])
        slider(disp, 60, np_y[i], 60, np_vals[i])
    disp.text('>', 50, np_y[mode_ptr])    # show the cursor on the active slider
    disp.show()
# Calculate the RGB levels
    for i in range(0,3):
        rgb[i] = int(max_lum * np_vals[i+1] / 100 * np_vals[0] / 100)
    np.fill((rgb[0],rgb[1],rgb[2]))    # show combined colour
    np.write()

# Clean up and exit
np.fill(black)
np.write()

