# File name: Pulse.py
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 4 January 2019
# Date last modified: 13 November 2019
# MicroPython Version: 1.10 for the Kookaberry V4.05
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# Measures and displays a pulse using the Gravity Heart Rate Sensor connected to plug P4
# 
#------------------------------------------
# Dependencies:
# I/O ports and peripherals: Gravity heart rate sensor
# /lib files: nil
# /root files: nil
# Other dependencies: nil
# Complementary apps: nil
#------------------------------------------# Begin code
import machine, kooka, fonts, time, framebuf
# two sample arrays are held - one of raw samples so auto-zero can be accomplished
# the second is scaled and limited for display purposes and pulse detection
samples = 40
sarray = [0] * samples
parray = [0] * samples
plotx = 0
ploty = 12
ploth = 40
plotw = 80
plotxinc = int(plotw/samples)
plotymid = int(ploty + ploth/2)
plotzero = 1600
plotrange = 600
pulse_d = 0    # digital pulse 0 if below hysteresis, 1 if above
pulse_h = int(plotrange/10)    # hysterisis limit
pulse_d_last = 0   # remember the previous digital pulse to detect change
tsamples = 4    # number of timer samples used to get a moving average pulse
pulse_t = [0] * tsamples
p_edge = 0    # indicator that a leading pulse edge has occurred
p_timer =[0] * 3    #array time now, time last, time interval
p_rate = 0

# Create a beating heart icon
bitmap = bytearray([0x80, 0xE0, 0xF0, 0xF8, 0xFC, 0xFC, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0x3C, 0x3C, 0xF8, 0xF8,
0xFC, 0xFC, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFC, 0xFC, 0xF8, 0xF0, 0xE0, 0x80, 0x1F, 0x7F,
0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x0F, 0x87, 0x1F, 0x1F, 0x01, 0xF0, 0x00, 0x0F, 0xFF, 0xFF, 0x7F,
0x7F, 0x7F, 0x0F, 0x0F, 0x3F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x1F, 0x00, 0x00, 0x00, 0x01,
0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x7E, 0xFC, 0xFC, 0xFF, 0xFE, 0x80, 0x03, 0xC0, 0xF8, 0xFC, 0xF8,
0x7C, 0x3F, 0x1F, 0x0F, 0x07, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x07, 0x0F, 0x0F, 0x07, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00])

heart = framebuf.FrameBuffer(bitmap, 30, 30, framebuf.MONO_VLSB)


# Instantiate the physical I/O
pulsepin = 'P4'
p4 = machine.ADC(machine.Pin(pulsepin))
disp = kooka.display
# The main loop
while not kooka.button_a.was_pressed():
    disp.fill(0)
    disp.setfont(fonts.mono8x8)
    disp.text('Pulse %s' % pulsepin, 0, 6)
    disp.setfont(fonts.mono6x7)
    disp.text('A-exit', 0, 60)
# Shift the data arrays left and get a fresh sample and scale it
    for i in range(0,samples-1):  
        parray[i]=parray[i+1]    # shift samples left
        sarray[i]=sarray[i+1]
    sarray[samples-1] = p4.read()
    plotzero = int(sum(sarray)/samples)    # zero line is the average of samples
    sample = p4.read()-plotzero
    sample = max(sample,-plotrange/2)
    sample = min(sample,plotrange/2)
    parray[samples-1] = int(sample)
# digitise pulse waveform and detect leading edge
    if pulse_d and parray[samples-1] < -pulse_h: pulse_d = 0
    if not pulse_d and parray[samples-1] > pulse_h: pulse_d = 1
    p_edge = 0
    if pulse_d and not pulse_d_last: p_edge = 1
    pulse_d_last = pulse_d
# measure the timing of pulse leading edges
    if p_edge:
        p_timer[0] = time.ticks_ms() # mark the time in ticks at the interrupt
        p_timer[2] = p_timer[0] - p_timer[1]  # interval = time now minus last time
        p_timer[1] = p_timer[0]
        for i in range (0,tsamples-1): pulse_t[i] = pulse_t[i+1]
        pulse_t[tsamples-1] = p_timer[2]    # load latest interval into array
        p_rate = sum(pulse_t)/tsamples    # get average interval
        p_rate = int(60000 / p_rate)    # pulse rate in beats per minute
# display the pulse rate
    disp.setfont(fonts.sans12)
    disp.text('%3d' % p_rate, 90,24)
    if pulse_d: disp.blit(heart, 90, 34)

    
# Display the pulse chart
    disp.rect(plotx,ploty,plotw,ploth+1,1)    # outline rectangle
    for i in range(0,samples-1):
        p1x = i*plotxinc+plotx
        p1y = plotymid-int(parray[i]/plotrange*ploth)
        p2x = (i+1)*plotxinc+plotx
        p2y = plotymid-int(parray[i+1]/plotrange*ploth)
        disp.line(p1x, p1y, p2x , p2y, 1)
    disp.show()

