# File name: RockPaperScissors.py
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 17 July 2018
# Date last modified: 19 Sep 2018
# MicroPython Version: 1.9.4 for the Kookaberry
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# Rock paper scissors game for Kookaberry
# Begin code

import time, kooka, fonts, math, random, framebuf
disp = kooka.display


# Initialise Scores
kscore = 0	# Kooka's score
uscore = 0	# User's score
choices = ["R","P","S"]	# Rock, Paper, Scissors symbols
kchoice = " "	# Kooka's selection generated randomly
uchoice = " "	# User's selection according to buttons
winner = " "	# Symbol indicating which player won the selection round
gameon = 1	# Flag whether game is active or over

# Create bitmaps for the implements
# Paper
papermap = framebuf.FrameBuffer(bytearray(32), 16, 16, framebuf.MONO_HLSB)
papermap.fill(0)
papermap.rect(1,1,14,14,1)
papermap.line(1,14,14,1,1)
# Rock
rockmap = framebuf.FrameBuffer(bytearray(32), 16, 16, framebuf.MONO_HLSB)
rockmap.fill(0)
rockmap.fill_rect(1,1,8,8,1)
rockmap.fill_rect(4,4,8,8,1)
rockmap.fill_rect(8,8,8,8,1)
rockmap.fill_rect(2,8,6,6,1)
# Scissors
scissormap = framebuf.FrameBuffer(bytearray(32), 16, 16, framebuf.MONO_HLSB)
scissormap.fill(0)
scissormap.fill_rect(1,12,4,4,1)
scissormap.fill_rect(12,12,4,4,1)
scissormap.line(1,14,14,1,1)
scissormap.line(1,1,14,14,1)
# User's choice
usermap = framebuf.FrameBuffer(bytearray(32), 16, 16, framebuf.MONO_HLSB)
usermap.fill(0)
# Kookaberry's choice
kookamap = framebuf.FrameBuffer(bytearray(32), 16, 16, framebuf.MONO_HLSB)
kookamap.fill(0)

while not kooka.button_a.was_pressed():	#Keep going until the A key is pressed

    disp.fill(0)	#Clear the display
    disp.setfont(fonts.mono6x7)	#Select small font
# Set up the fixed text on the screen
    disp.text('Rock Paper Scissors', 0, 6)
    disp.text('Kooka',0,15)
    disp.text('You', 108, 15)
    disp.text('X', 16, 57)	#A Key = Exit
    disp.text('R', 48, 57)	#C Key = Rock
    disp.text('P', 80, 57)	#D Key = Paper
    disp.text('S', 112, 57)	#B Key = Scissors
# Display scores
    disp.setfont(fonts.sans12)
    disp.fill_rect(0, 20, 18, 24, 1)
    disp.text('%u' % kscore, 2, 38, 0)
    disp.fill_rect(110, 20, 128, 24, 1)
    disp.text('%u' % uscore, 112, 38, 0)
# Display last choices and result indicator
#    disp.text('%s' % kchoice, 22, 38, 1)
#    disp.text('%s' % uchoice, 90, 38, 1)
    disp.text('%s' % winner, 56, 38, 1)
    disp.blit(kookamap, 22, 24)
    disp.blit(usermap, 90, 24)
# Show the screen
    disp.show()
# If maximum score reached then stop checking user buttons and reset the game after a delay
    if gameon == 0: 
	time.sleep_ms(5000)
	kscore = 0
	uscore = 0
	kchoice = " "
	uchoice = " "
	winner = " "
	gameon = 1
	continue
# Check for user choice
    if kooka.button_b.was_pressed():	#User chose Scissors
        uchoice = choices[2]
        kchoice = choices[random.randint(0,2)]
        winner = "="
        usermap = scissormap
        kookamap = scissormap
        if kchoice == "P": 
            uscore += 1
            winner = "<"
            kookamap = papermap                        
        if kchoice == "R": 
            kscore += 1
            winner = ">"
            kookamap = rockmap
    if kooka.button_c.was_pressed():	#User chose Rock
        uchoice = choices[0]
        kchoice = choices[random.randint(0,2)]
        winner = "="
        usermap = rockmap
        kookamap = rockmap
        if kchoice == "S": 
            uscore += 1
            winner = "<"
            kookamap = scissormap
        if kchoice == "P": 
            kscore += 1
            winner = ">"
            kookamap = papermap
    if kooka.button_d.was_pressed():	#User chose Paper
        uchoice = choices[1]
        kchoice = choices[random.randint(0,2)]
        winner = "="
        usermap = papermap
        kookamap = papermap
        if kchoice == "R": 
            uscore += 1
            winner = "<"
            kookamap = rockmap
        if kchoice == "S": 
            uscore += 1
            winner = ">"
            kookamap = scissormap
# Check whether to stop the game
    if kscore >= 9: gameon = 0
    if uscore >= 9: gameon = 0
# Small delay before polling again
    time.sleep_ms(50)