# File name: Scope.py
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 27 November 2019
# Date last modified: 27 November 2019
# MicroPython Version: 1.10 for the Kookaberry V4.05
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# Measures and displays time series analogue readings from any sensor connected to plug P4
# Controls are provided to vary the sample intervals between display updates
# (This is because drawing the display takes a long time)
# Button C halves the update interval, button D doubles it, between limits
# The display shows a sample every two pixels on the X axis - default is 64 samples
# 
#------------------------------------------
# Dependencies:
# I/O ports and peripherals: Analogue sensor plugged into P4
# /lib files: nil
# /root files: nil
# Other dependencies: nil
# Complementary apps: nil
#------------------------------------------
# Begin code
import machine, kooka, fonts, time, framebuf
# two sample arrays are held - one of raw samples so auto-zero can be accomplished
# the second is scaled and limited for display purposes and pulse detection
samples = 64
sarray = [0] * samples    # sensor array
parray = [0] * samples    # plotting array
tarray = [0] * samples    # sample timing array
plotx = 0
ploty = 12
ploth = 40
plotw = 128
plotxinc = int(plotw/samples)
plotymid = int(ploty + ploth/2)
plotzero = 2047
plotrange = 2047
disp_samples = 8    # Interval between plots
disp_min = 1
disp_max = samples
disp_ctr = 0
disp_freeze = False
p_timer =[0] * 3    #array time now, time last, time interval
p_timer[1] = time.ticks_ms() # mark the time in ticks

# Instantiate the physical I/O
pulsepin = 'P4'
input = machine.ADC(machine.Pin(pulsepin))
disp = kooka.display
# The main loop
while not kooka.button_a.was_pressed():
# Shift the data arrays left and get a fresh sample and scale it
    for i in range(0,samples-1):  
        parray[i]=parray[i+1]    # shift samples left
        sarray[i]=sarray[i+1]
        tarray[i]=tarray[i+1]
    sarray[samples-1] = input.read()
    plotzero = int(sum(sarray)/samples)    # zero line is the average of samples
    sample = input.read()-plotzero
    sample = max(sample,-plotrange/2)
    sample = min(sample,plotrange/2)
    parray[samples-1] = int(sample)
    tarray[samples-1] = p_timer[2]
    disp_ctr -= 1    # count down the samples
    
# Adjust the display timing parameters
    if kooka.button_c.was_pressed():    # decrease the bounce capture period
        disp_samples = int(max(disp_min, disp_samples / 2))
    if kooka.button_d.was_pressed():    # increase the bounce capture period
        disp_samples = int(min(disp_max, disp_samples * 2))
    if kooka.button_b.was_pressed():    # increase the bounce capture period
        disp_freeze = not disp_freeze
# Measure loop timing
    p_timer[0] = time.ticks_ms() # mark the time in ticks now
    p_timer[2] = p_timer[0] - p_timer[1]  # interval = time now minus last time
    p_timer[1] = p_timer[0]

# Static display
    if disp_ctr <= 0 and not disp_freeze:
        tsum = 0
        for i in range(0,len(tarray)): tsum += tarray[i]
        tavg = tsum/len(tarray)
        disp.fill(0)
        disp.setfont(fonts.mono8x8)
        disp.text('Scope %s %d msec' % (pulsepin,tavg), 0, 6)
        disp.setfont(fonts.mono6x7)
        disp.text('A-ex  C- %2d D+  B-Frz '% disp_samples, 0, 60)

# Display the chart
        disp.rect(plotx,ploty,plotw,ploth+1,1)    # outline rectangle
        for i in range(0,samples-1):
            p1x = i*plotxinc+plotx
            p1y = plotymid-int(parray[i]/plotrange*ploth)
            p2x = (i+1)*plotxinc+plotx
            p2y = plotymid-int(parray[i+1]/plotrange*ploth)
            disp.line(p1x, p1y, p2x , p2y, 1)
        disp.show()
        disp_ctr = disp_samples

    

