# File name: 
__name__ = 'SeeTemp'
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 7 April 2020
# Date last modified: 7 April 2020
# MicroPython Version: 1.12 for the Kookaberry V4-05
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# Measure spot temperature using a non-contact infra-red digital temperature sensor when a button is pressed
# Logs the measurements into a file when a measurement is taken.
# If the sensor is not present prompt is displayed. 
#------------------------------------------
# Dependencies:
# I/O ports and peripherals: GY-906 probe plugged into P3
# /lib files: kcpu.py, mlx90614.py
# /root files: Nil
# Other dependencies: Nil
# Complementary apps: GraphCSV used to convert the logged data into a graph
#------------------------------------------
# Begin code
# Initial conditions

import machine, kooka, fonts, time
import mlx90614
import kcpu
disp = kooka.display    # initialise the OLED display
dspin = 'P3'

# Startup display
disp.fill(0)
disp.setfont(fonts.mono8x8)
disp.text('%s Sens:%s' % (__name__,dspin), 0, 6)
disp.setfont(fonts.mono6x7)
disp.text('Initialising', 0,17)
disp.text('Connect sensor', 0,26)
disp.text('to socket %s' % dspin, 0,35)
disp.text('A-exit', 0, 60)
disp.show()

# set up the datalogger
fname = __name__+'.csv'
f = open(fname,'w+')    # Create the datalogger instance
f.write('Reading, Ambient-degC, Object-degC\n')
f.close()
# Initialise the sensor
i2c = machine.I2C(scl=machine.Pin('P3B'), sda=machine.Pin('P3A'), freq=50000)
s_flag = False
while not s_flag and not kooka.button_a.was_pressed() and kcpu.battery_ok():    # sensor doesn't always talk at first - retry loop
    try:
        sensor = mlx90614.MLX90614(i2c)
        s_flag = True
    except:
        print('Sensor fail')
        time.sleep_ms(500)
read_no = 0
temp = 0
otemp = 0
stemp = 0
t_read = False
t_fail = False

while not kooka.button_a.was_pressed() and kcpu.battery_ok():
# Read and process the sensor, send the radio message, update the logger
    if kooka.button_b.was_pressed():
        t_read = False
        try:
            kooka.led_green.off()
            kooka.led_red.off()
            kooka.led_orange.on()
            temp = sensor.read_ambient_temp()
            otemp = sensor.read_object_temp()
            if sensor.dual_zone:
                stemp = sensor.object2_temp 
            t_read = True
            t_fail = False
            read_no += 1
            kooka.led_orange.off()
            kooka.led_green.on()
            print(t_read, read_no, temp, otemp)
            f = open(fname,'a+')    # Create the datalogger instance
            f.write('%d, %.2f, %.2f\n' % (read_no, temp, otemp))
            f.close()

        except:
            t_fail = True
            kooka.led_orange.off()
            kooka.led_red.on()
            pass
# Prepare the display
    disp.fill(0)
    disp.setfont(fonts.mono8x8)
    disp.text('%s Sens:%s' % (__name__,dspin), 0, 6)
    disp.setfont(fonts.mono6x7)
    disp.text('A-exit', 0, 60)
    disp.text('B-measure', 65, 60)
    disp.setfont(fonts.mono6x7)
    if t_read: 
        disp.text('Ambient: %0.2f C' % (temp), 12, 17)
        disp.text('Read No: %d' % (read_no), 12, 26)
        disp.setfont(fonts.sans12)
        disp.text('%0.2f C' % otemp, 12, 47)
    elif read_no == 0 and not t_fail:    # No reading attempted as yet
        disp.text('Point sensor &', 0,17)
        disp.text('press button B', 0,26)
        disp.setfont(fonts.sans12)
        disp.text('?.? C', 12, 47)
    elif t_fail:
        disp.text('Sensor failure', 0,17)
        disp.text('Reconnect to %s' % dspin, 0,26)
        disp.setfont(fonts.sans12)
        disp.text('?.? C', 12, 47)
    disp.show()

        
# Clean up and exit


